package com.bpcl.lpg.core.service.impl;

import org.slf4j.LoggerFactory;
import java.text.MessageFormat;
import org.apache.commons.lang.StringUtils;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.core.JsonProcessingException;
import java.util.Objects;
import java.net.http.HttpResponse;
import com.bpcl.lpg.facades.data.FetchPaymentDetailsRequestData;
import com.bpcl.lpg.core.service.BpclHttpClientService;
import de.hybris.platform.servicelayer.config.ConfigurationService;
import org.slf4j.Logger;
import com.bpcl.lpg.core.service.LpgFetchPaymentDetailsService;

public class LpgFetchPaymentDetailsServiceImpl implements LpgFetchPaymentDetailsService
{
    private static final Logger LOG;
    private ConfigurationService configurationService;
    private BpclHttpClientService bpclHttpClientService;
    
    public HttpResponse<String> getPaymentResponse(final FetchPaymentDetailsRequestData requestData, final String requestId, final String paymentEndPointName) throws JsonProcessingException {
        Objects.requireNonNull(paymentEndPointName, "End point name should not be null");
        final String endPoint = this.getEndPoint(paymentEndPointName, requestId);
        final String jsonBody = this.getRequestJson(requestData, endPoint);
        return this.getBpclHttpClientService().executePostMethod(endPoint, jsonBody, paymentEndPointName);
    }
    
    private String getRequestJson(final FetchPaymentDetailsRequestData requestData, final String endPoint) throws JsonProcessingException {
        final ObjectMapper objectMapper = new ObjectMapper();
        final String json = objectMapper.writerWithDefaultPrettyPrinter().writeValueAsString(requestData);
        return objectMapper.writeValueAsString(requestData);
    }
    
    private String getEndPoint(final String paymentEndPointName, final String requestId) {
        final String endpointUri = this.getConfigurationService().getConfiguration().getString(paymentEndPointName);
        Objects.requireNonNull(endpointUri, "Endpoint should not be null");
        return StringUtils.isBlank(requestId) ? endpointUri : MessageFormat.format(endpointUri, requestId);
    }
    
    public ConfigurationService getConfigurationService() {
        return this.configurationService;
    }
    
    public void setConfigurationService(final ConfigurationService configurationService) {
        this.configurationService = configurationService;
    }
    
    public BpclHttpClientService getBpclHttpClientService() {
        return this.bpclHttpClientService;
    }
    
    public void setBpclHttpClientService(final BpclHttpClientService bpclHttpClientService) {
        this.bpclHttpClientService = bpclHttpClientService;
    }
    
    static {
        LOG = LoggerFactory.getLogger(LpgFetchPaymentDetailsServiceImpl.class);
    }
}
